#!/usr/bin/env python3

import re
import sys
import glob
from datetime import datetime, timedelta

def format_delta(seconds_float):
    sign = "-" if seconds_float < 0 else "+"
    total_ms = abs(int(round(seconds_float * 1000)))

    days, rem = divmod(total_ms, 86400000)
    hours, rem = divmod(rem, 3600000)
    minutes, rem = divmod(rem, 60000)
    seconds, milliseconds = divmod(rem, 1000)

    parts = []
    if days > 0:
        parts.append(f"{days}d")
    if hours > 0 or days > 0:
        parts.append(f"{hours}h")
    if minutes > 0 or hours > 0 or days > 0:
        parts.append(f"{minutes}m")
    parts.append(f"{seconds}s {milliseconds}ms")

    return sign + " ".join(parts)

def process_file(filepath):
    with open(filepath, "r", encoding="utf-8") as f:
        raw_lines = f.readlines()

    re_log_time = re.compile(r'\[([0-9]{2}\.[0-9]{2}\.[0-9]{4} [0-9]{2}:[0-9]{2}:[0-9]{2}\.[0-9]{3})')
    re_timestamp = re.compile(r'"timestamp":\s*([0-9]+)')

    parsed_lines = []

    for line in raw_lines:
        m1 = re_log_time.search(line)
        if m1:
            parsed_lines.append(m1.group(1))
            continue
        m2 = re_timestamp.search(line)
        if m2:
            parsed_lines.append(m2.group(1))
            continue

    results = []
    i = 0
    while i < len(parsed_lines) - 1:
        log_time_str = parsed_lines[i].strip()
        ts_str = parsed_lines[i + 1].strip()

        if not ts_str.isdigit():
            i += 1
            continue

        ts = int(ts_str)
        ts_local = datetime.fromtimestamp(ts)
        ts_utc = ts_local - timedelta(hours=3)

        try:
            log_time_utc = datetime.strptime(log_time_str, "%d.%m.%Y %H:%M:%S.%f")
        except ValueError:
            i += 2
            continue

        delta_sec = (log_time_utc - ts_utc).total_seconds()
        delta_human = format_delta(delta_sec)

        # Добавляем как tuple, без цвета — для сортировки
        results.append((
            log_time_str,
            ts,
            ts_utc.strftime('%Y-%m-%d %H:%M:%S'),
            delta_sec,
            delta_human
        ))

        i += 2

    return results

def main():
    if len(sys.argv) < 2:
        print("Usage: python3 analyze_log_timestamps.py <path/mask>")
        sys.exit(1)

    all_results = []

    for pattern in sys.argv[1:]:
        for filepath in glob.glob(pattern):
            all_results.extend(process_file(filepath))

    # Сортируем по первому столбцу (дата из лога)
    all_results.sort(key=lambda row: row[0])

    for log_time_str, ts, ts_utc_str, delta_sec, delta_human in all_results:
        result_line = f"{log_time_str} | {ts} | {ts_utc_str} | {delta_sec:+.3f} | {delta_human}"

        # Цветовая подсветка
        if abs(delta_sec) > 3600:
            color = "\033[31m"  # Красный
        elif abs(delta_sec) > 300:
            color = "\033[33m"  # Жёлтый
        else:
            color = "\033[0m"   # Сброс

        print(f"{color}{result_line}\033[0m")

if __name__ == "__main__":
    main()
