#!/usr/bin/env bash
#
# modbus_poll.sh — скрипт-обёртка вокруг modbus_client.
# ──────────────────────────────────────────────────────────
# Поддерживает:
#   • выбор скорости (-B), чётности (-P) и стоп-бит (-S);
#   • опцию --no-service (не трогать wb-mqtt-serial);
#   • вывод параметров соединения в виде красивой таблицы
#     через Python-библиотеку tabulate (fallback — инструкция).
#   • корректное заполнение таблицы значений (N/A при ошибках).
#
# Использование
#   ./modbus_poll.sh [ОПЦИИ] <bus_no> <slave_addr> <reg>
#   ./modbus_poll.sh [ОПЦИИ] <bus_no> <slave_addr> <start_reg> <end_reg>
#
# Пример
#   ./modbus_poll.sh -B9600 -P even -S1 2 3 4000 4010
# ──────────────────────────────────────────────────────────

set -euo pipefail

########################  дефолты
BAUD=19200
PARITY=none           # none|even|odd
STOPBITS=2            # 1|2
SERVICE_CTRL=1        # 0 = не трогать wb-mqtt-serial

########################  парсинг опций
print_help(){ sed -n '1,/^$/p' "$0"; }

while [[ $# -gt 0 ]]; do
  case "$1" in
    -B) BAUD="$2";     shift 2 ;;
    -P) PARITY="$2";   shift 2 ;;
    -S) STOPBITS="$2"; shift 2 ;;
    --no-service) SERVICE_CTRL=0; shift ;;
    -h|--help) print_help; exit 0 ;;
    --) shift; break ;;
    -*) echo "Неизвестная опция: $1" >&2; print_help; exit 1 ;;
    *)  break ;;
  esac
done

########################  позиционные аргументы
if [[ $# -lt 3 || $# -gt 4 ]]; then
  echo "Неверное количество аргументов." >&2
  print_help; exit 1
fi

BUS_NUM=$1
ADDRESS=$2
START=$3
END=${4:-$3}
BUS_DEV="/dev/ttyRS485-${BUS_NUM}"

########################  массивы кодов/названий
TYPES=(0x01 0x02 0x03 0x04)
TYPE_NAMES=(coil discrete holding input)

########################  функция: таблица параметров
print_config_table() {
  # передаём переменные в окружение дочернего процесса
  BAUD="$BAUD" PARITY="$PARITY" STOPBITS="$STOPBITS" \
  BUS_DEV="$BUS_DEV" ADDRESS="$ADDRESS" START="$START" END="$END" \
  python3 - <<'PY'
import os, sys, textwrap

rows = [
    ["Скорость (baud)",       os.environ["BAUD"]],
    ["Чётность (parity)",     os.environ["PARITY"]],
    ["Стоп-биты",             os.environ["STOPBITS"]],
    ["Интерфейс",             os.environ["BUS_DEV"]],
    ["Modbus-адрес (slave)",  os.environ["ADDRESS"]],
    ["Диапазон регистров",    f"{os.environ['START']} – {os.environ['END']}"],
]

YEL = "\033[1;33m"; RESET = "\033[0m"

try:
    from tabulate import tabulate
    print(YEL + tabulate(rows, headers=["Параметр", "Значение"],
                         tablefmt="fancy_grid") + RESET)
except ModuleNotFoundError:
    print(f"{YEL}[⚠] Библиотека 'tabulate' не найдена.{RESET}")
    print("    Установите её:\n"
          "      pip3 install tabulate\n"
          "    или\n"
          "      sudo apt install python3-tabulate\n")
    print("Параметры соединения:")
    for k, v in rows:
        print(f"  • {k}: {v}")
PY
  echo
}

########################  вывод параметров
print_config_table

########################  заголовок таблицы значений
printf "%10s" "Регистр"
for n in "${TYPE_NAMES[@]}"; do printf "%20s" "$n"; done
echo
printf "%10s" "----------"
for _ in "${TYPE_NAMES[@]}"; do printf "%20s" "--------------------"; done
echo

########################  остановка wb-mqtt-serial
(( SERVICE_CTRL )) && service wb-mqtt-serial stop

########################  опрос
for ((reg = START; reg <= END; reg++)); do
  printf "%10d" "$reg"
  for fcode in "${TYPES[@]}"; do
    RAW=$(modbus_client --debug -mrtu -p"$PARITY" -b"$BAUD" -s"$STOPBITS" \
                        "$BUS_DEV" -a"$ADDRESS" -t"$fcode" -r"$reg" -c1 \
                        2>/dev/null || true)
    RES=$(awk '/Data:/ {sub(/^Data:[[:space:]]*/,""); print}' <<<"$RAW")
    [[ -z "$RES" ]] && RES="N/A"
    printf "%20s" "$RES"
    sleep 0.1
  done
  echo
done

########################  запуск wb-mqtt-serial
(( SERVICE_CTRL )) && service wb-mqtt-serial start

